@@ -22,7 +22,8 @@ def maximize_number_of_file_descriptors():
         import win32file
         win32file._setmaxstdio(2048)
 
-def quality_report(analytics, df, rep=None):
+# If DF is not supplied, no histograms are generated.
+def quality_report(analytics, df=None, rep=None):
     maximize_number_of_file_descriptors()
     # Need this so it finds custom fonts
     font_manager._rebuild()
@@ -35,7 +36,7 @@ def quality_report(analytics, df, rep=None):
     rcParams['xtick.color'] = '#DEE2EA'
     rcParams['ytick.color'] = '#DEE2EA'
 
-    if not rep:
+    if not isinstance(rep, pd.DataFrame):
         rep = analytics.report(df)
 
     here_dir = os.path.dirname(os.path.realpath(__file__))
@@ -91,7 +92,7 @@ def generate_html(analytics, df, rep, all_stats):
             cover(doc)
             page_break(doc)
             header(doc)
-            product_stats_section(analytics, rep, all_stats, doc)
+            product_stats_section(analytics, df, rep, all_stats, doc)
             page_break(doc)
             header(doc)
             data_stats_section(analytics, rep, doc)
@@ -122,7 +123,7 @@ def cover(doc):
 def legend_item(doc, color):
     doc.line("span", NBSP, klass="legend_item", style="background-color: " + color)
 
-def product_stats_section(a, rep, all_stats, doc):
+def product_stats_section(a, df, rep, all_stats, doc):
     doc.line("h2", "Data Products")
     # Legend
     with doc.tag("div", klass="product_stats_legend"):
@@ -158,9 +159,21 @@ def product_stats_section(a, rep, all_stats, doc):
                             doc.line("td", nbsp("\u25B6 Overall Results for this Product: "))
                             with doc.tag("td"):
                                 product_stat_bar(doc, pstats["product_match_rate"].iloc[0], pstats["product_error_rate"].iloc[0])
+    doc.line("h3", "More Statistics")
+    with doc.tag("table"):
+        if isinstance(df, pd.DataFrame):
+            with doc.tag("tr"):
+                doc.line("td", "\u25B6 Number of Records: ")
+                doc.line("td", str(len(df)))
+        with doc.tag("tr"):
+            doc.line("td", "\u25B6 Number of Products: ")
+            doc.line("td", str(len(provider_names_from_report(rep))))
+        with doc.tag("tr"):
+            doc.line("td", "\u25B6 Records that Match All Products: ")
+            doc.line("td", str(rep["matches_all"].values[0]))
 
 def product_stats_from_all_stats(all_stats, pid):
-    return all_stats.loc[all_stats["product"] == pid].sort_values("stats_updated_on", ascending=False)
+    return all_stats.loc[all_stats["product_name"] == pid].sort_values("stats_updated_on", ascending=False)
 
 def product_stat_bar(doc, match_rate, error_rate):
     with doc.tag("table", klass="product_stats_diagram", cellpadding="0", cellspacing="0"):
@@ -182,7 +195,8 @@ def distribution_diagrams(a, rep, all_stats):
         # per-job hist
         numattr = len(rep.loc[rep['product_name'] == p])
         x = rep.loc[rep['product_name'] == p]['attribute_fill_rate'].to_numpy()
-        arr = plt.hist(x)
+        arr = plt.hist(x, 100)
+        plt.gcf().set_size_inches(5, 2)
         plt.xlabel('Fill Rate (%)')
         plt.ylabel('# of Attributes', rotation='90')
         plt.tight_layout()
@@ -190,11 +204,12 @@ def distribution_diagrams(a, rep, all_stats):
         # overall hist
         pstats = product_stats_from_all_stats(all_stats, p)
         if not pstats.empty:
-            pstats['attribute_flattened_name'] = pstats['attribute_flattened_name'].str.replace(r"\[(\d+)\]", '[*]', regex=True)
-            pstats = pstats.drop_duplicates(subset="attribute_flattened_name")
-            numattr = len(pstats.loc[pstats['product'] == p])
-            x = pstats.loc[pstats['product'] == p]['attribute_fill_rate'].to_numpy() * 100.0
-            arr = plt.hist(x)
+            pstats['attribute_name'] = pstats['attribute_name'].str.replace(r"\[(\d+)\]", '[*]', regex=True)
+            pstats = pstats.drop_duplicates(subset="attribute_name")
+            numattr = len(pstats.loc[pstats['product_name'] == p])
+            x = pstats.loc[pstats['product_name'] == p]['attribute_fill_rate'].to_numpy() * 100.0
+            arr = plt.hist(x, 100)
+            plt.gcf().set_size_inches(5, 2)
             plt.xlabel('Fill Rate (%)')
             plt.ylabel('# of Attributes', rotation='90')
             plt.tight_layout()
@@ -258,30 +273,31 @@ def details_section(a, df, rep, all_stats, doc):
                 doc.line("h4", "\u25B6 Overall Results for this Product: ")
                 doc.stag("img", src=pid + "_fill2.svg", style="width: 50%")
             attrs = rep.loc[(rep['product_name'] == pid)]
-            pstats['attribute_flattened_name'] = pstats['attribute_flattened_name'].str.replace(r"\[(\d+)\]", '[*]', regex=True)
-            pstats = pstats.drop_duplicates(subset="attribute_flattened_name")
+            pstats['attribute_name'] = pstats['attribute_name'].str.replace(r"\[(\d+)\]", '[*]', regex=True)
+            pstats = pstats.drop_duplicates(subset="attribute_name")
             doc.line("h3", "Attributes")
             for idx, row in attrs.iterrows():
                 with doc.tag("div", style="page-break-inside: avoid"):
                     doc.line("h4", row["product_name"] + "." + row["attribute_name"], klass="attr_name")
                     with doc.tag("table", style="width: 100%"):
                         with doc.tag("tr"):
-                            doc.line("td", nbsp("\u25B6 Results for this Job: "), style="width: 10%")
-                            with doc.tag("td"):
+                            doc.line("td", nbsp("\u25B6 Results for this Job: "), klass="attr_field")
+                            with doc.tag("td", klass="attr_field_value"):
                                 render_attr_bar(doc, row["attribute_fill_rate"])
                         if not pstats.empty:
-                            astats = pstats.loc[(pstats['attribute_flattened_name'] == row["attribute_name"])]
+                            astats = pstats.loc[(pstats['attribute_name'] == row["attribute_name"])]
                             if not astats.empty:
                                 astat = astats.iloc[0]
                                 with doc.tag("tr"):
-                                    doc.line("td", nbsp("\u25B6 Overall Results for this Attribute: "), style="width: 10%")
-                                    with doc.tag("td"):
+                                    doc.line("td", nbsp("\u25B6 Overall Results for this Attribute: "), klass="attr_field")
+                                    with doc.tag("td", klass="attr_field_value"):
                                         render_attr_bar(doc, astat["attribute_fill_rate"] * 100.0)
-#                     if row["attribute_type"] == np.dtype("float64"):
-#                         with doc.tag("tr"):
-#                             with doc.tag("td"):
-#                                 attr_hist_diagram(row, df, pid, idx)
-#                                 doc.stag("img", src=pid + "_hist_" + str(idx) + ".svg")
+                        if isinstance(df, pd.DataFrame) and (row["attribute_type"] == np.dtype("float64")):
+                            with doc.tag("tr"):
+                                doc.line("td", nbsp("\u25B6 Attribute Histogram: "), klass="attr_field")
+                                with doc.tag("td", klass="attr_field_value"):
+                                    attr_hist_diagram(row, df, pid, idx)
+                                    doc.stag("img", src=pid + "_hist_" + str(idx) + ".svg")
 #                    with doc.tag("tr"):
 #                        doc.line("td", nbsp("Unique Values: "), style="width: 10%")
 #                        doc.line("td", row["unique_values"])
@@ -289,48 +305,14 @@ def details_section(a, df, rep, all_stats, doc):
             header(doc)
 
 def attr_hist_diagram(row, df, pid, idx):
-    rcParams['font.family'] = 'Roboto Mono'
-    rcParams['font.weight'] = '400'
-    rcParams['text.color'] = '#a1adc4'
-    rcParams['axes.labelcolor'] = '#a1adc4'
-    rcParams['xtick.color'] = '#a1adc4'
-    rcParams['ytick.color'] = '#a1adc4'
     # Note: uses only first element of list attr
     attr_name = row['attribute_name'].replace('[*]', '[0]')
     full_name = row['product_name'] + '.' + attr_name
     x = df[full_name].to_numpy()
-    plt.grid(b=False)
-    plt.box(on=False)
-    num_bins = 50
-    plt.yticks(ticks=[], labels=[])
     plt.gcf().set_size_inches(5, 2)
-
-    try:
-        sd = np.std(x)
-        n, bins, patches = plt.hist(x, num_bins, density=1)
-    except (TypeError, ValueError) as e:
-        print(full_name)
-        plt.clf()
-        plt.text(0.5, 0.5, 'Not Really a Number', horizontalalignment='center', verticalalignment='center')
-        return plt
-    n, bins, patches = plt.hist(x, num_bins, density=1)
-    sd = np.nanstd(x)
-    m = np.nanmean(x)
-    sdl = m - (2 * sd)
-    sdh = m + (2 * sd)
-    plt.axvline(m, color='b', linestyle='solid', linewidth=1)
-    plt.axvline(sdl, color='b', linestyle='solid', linewidth=1)
-    plt.axvline(sdh, color='b', linestyle='solid', linewidth=1)
-    _, max_ = plt.ylim()
-    plt.text(m + m / 10,
-             max_ - max_ / 10,
-             'Mean: {:.2f}'.format(m))
-    plt.text(sdl + m / 10,
-             max_ - (2 * max_ / 10),
-             '-2sd: {:.2f}'.format(sdl))
-    plt.text(sdh + m / 10,
-             max_ - (2 * max_ / 10),
-             '+2sd: {:.2f}'.format(sdh))
+    plt.hist(x)
+    plt.xlabel('Numerical Value')
+    plt.ylabel('# of Rows', rotation='90')
     plt.tight_layout()
     save_figure(pid + "_hist_" + str(idx))
 
